import numpy as np
from scipy import interpolate
import pandas as pd
import h5py

import corner
import matplotlib.pyplot as plt
from matplotlib import rcParams
import matplotlib.lines as mlines
rcParams["text.usetex"] = True
rcParams["font.serif"] = "Computer Modern Roman"
rcParams["font.family"] = "Serif"
rcParams["xtick.labelsize"]=20
rcParams["ytick.labelsize"]=20
rcParams["legend.fontsize"]=17
rcParams["axes.labelsize"]=20


def read_samples(filename):
    with h5py.File(filename, "r") as ff:
        data = ff["samples"][:]
        events = [event.decode() for event in ff["events"]]
    data_frame = pd.DataFrame()
    for name in data.dtype.names:
        data_frame[name] = data[name]
    return data_frame, events


astro_ppd_samples = "o1o2o3_mass_c_iid_mag_two_comp_iid_tilt_powerlaw_redshift_ppd_45_event_samples_excludes_190814.json"
astro_ppd_samples_all_events = "o1o2o3_mass_c_iid_mag_two_comp_iid_tilt_powerlaw_redshift_ppd_45_event_samples_all_events.json"
observed_ppd_samples = "o1o2o3_mass_c_iid_mag_two_comp_iid_tilt_powerlaw_redshift_full_posterior_draw_45_events_excludes_190814.hdf5"
observed_ppd_samples_all_events = "o1o2o3_mass_c_iid_mag_two_comp_iid_tilt_powerlaw_redshift_full_posterior_draw_45_events_all_events.hdf5"


observed_samples = [observed_ppd_samples, observed_ppd_samples_all_events]

# Read in observed data

def read_observed_data(fname):
    synthetic_data = []
    data, events = read_samples(fname)
    # ignoring number of events using fixed number instead
    n_events = 45
    parameters = [
        "_".join(key.split("_")[1:]) for key in data
        if events[0] in key and "prior" not in key
    ]

    synthetic = {
        key: np.array([
            data[f"synthetic_{key}_{ii}"] for ii in range(n_events)
            if f"synthetic_{key}_{ii}" in data
        ])
        for key in parameters
        if any([f"synthetic_{key}_{ii}" in data for ii in range(n_events)])
    }

    synthetic["mass_2"] = synthetic["mass_1"]*synthetic["mass_ratio"]
    return synthetic

synthetic1 = read_observed_data(observed_ppd_samples)
synthetic2 = read_observed_data(observed_ppd_samples_all_events)

#Read in astrophysical data

data1 = pd.read_json(astro_ppd_samples)
data2 = pd.read_json(astro_ppd_samples_all_events)



# Plot m1 m2 ppd data

data1["mass_2"]= data1["mass_1"]*data1["mass_ratio"]
data2["mass_2"]= data2["mass_1"]*data2["mass_ratio"]

print("plotting m1 m2 distribution")

if np.isinf(data1["mass_ratio"].values).any():
    print("True inf")
if np.isnan(data1["mass_ratio"].values).any():
    print("True nan")

corner.hist2d(
    data1["mass_1"].values, data1["mass_2"].values, 
    levels=[0.99], plot_density=False, 
    contour_kwargs = {'linewidths':0},
    fill_contours=True,plot_datapoints=False,
    no_fill_contours=True,  smooth=0.8, 
    contourf_kwargs = {'alpha':0.3,'colors':["white","#2592db"]} 
)
corner.hist2d(
    data2["mass_1"].values, data2["mass_2"].values,
    levels=[0.99], plot_density=False,
    contour_kwargs = {'linewidths':0},
    fill_contours=True,plot_datapoints=False, 
    smooth=0.80, no_fill_contours=True, 
    contourf_kwargs = {'alpha':0.3, 'colors':["white","#154d73"]}
)

corner.hist2d(synthetic1["mass_1"],synthetic1["mass_2"],
              levels=[0.99], plot_density=False,
              no_fill_contours=True,plot_datapoints=False,
              color="#2592db", smooth=0.8, 
              contour_kwargs = {'linewidths':2}
             )
corner.hist2d(synthetic2["mass_1"],synthetic2["mass_2"],
              levels=[0.99], plot_density=False,
              no_fill_contours=True,plot_datapoints=False,
              color="#154d73", smooth=0.8, 
              contour_kwargs = {'linewidths':2}
             )

light_blue_line = mlines.Line2D([], [], color='#2592db', label='Excluding GW190814')
dark_blue_line = mlines.Line2D([], [], color='#154d73', label='Including GW190814')

plt.fill_between(np.linspace(2,100,100),np.linspace(2,100,100),100*np.ones(100),color='gray',zorder=2)
plt.fill_between(np.linspace(2,100,10),0.5,2,color='white',zorder=2)
plt.xlabel(r'$m_1 \ [M_\odot]$')
plt.ylabel(r'$m_2 \ [M_\odot]$')
plt.xlim(2,100)
plt.ylim(0,100)
plt.plot(22.9,2.6,marker='x', markersize=10, markeredgewidth=2, color="black")
plt.plot(np.linspace(2,100,10),2*np.ones(10),linewidth=0.1, color='lightgrey')
plt.legend(handles=[light_blue_line,dark_blue_line], loc='upper left',framealpha=1)

plt.tight_layout()
plt.savefig("m1m2_powerlaw_peak.png",dpi=800)
plt.close()


# Read data for observed minimum secondary mass and mass ratio distribution

all_m2_samples_1 = []
all_m2_samples_2 = []
for j in range(5000):
    m2_samples1 = []
    m2_samples2 = []
    values1 = synthetic1["mass_2"]
    values2 = synthetic2["mass_2"]
    for i in range(45):
        samples1 = values1[i]
        samples2 = values2[i]
        sample1 = samples1[j]
        sample2 = samples2[j]
        m2_samples1.append(sample1)
        m2_samples2.append(sample2)
    all_m2_samples_1.append(np.min(m2_samples1))
    all_m2_samples_2.append(np.min(m2_samples2)) 


# Plot data for observed minimum secondary mass distribution

print("Plotting observed minimum secondary mass distribution")

plt.hist(all_m2_samples_1,bins=40,histtype='bar',
         density=True,color="#2592db",linewidth=3,
         alpha=0.5,rwidth=0.95,label='Excluding GW190814'
        )

plt.hist(all_m2_samples_2,bins=40,histtype='bar',
         density=True,color="#154d73",linewidth=3,
         alpha=0.5,rwidth=0.95,label='Including GW190814'
        )

y1 = np.linspace(0,0.8,3)
x1 = np.array([2.6,2.6,2.6])

plt.plot(x1,y1, linestyle='dashed',linewidth=2, color='black')
plt.axvspan(2.5, 2.7, alpha=0.4, color='black')

plt.ylabel(r'$p(\mathrm{min}_{45}(m_2)$')
plt.xlabel(r'$\mathrm{min}_{45}(m_2) \ [M_\odot]$')
plt.xlim(2,12)
plt.ylim(0,0.8)
plt.legend(frameon=False)
plt.tight_layout()
plt.savefig("min_m2_powerlaw_peak_weighted.png",dpi=800)
